from distutils.version import LooseVersion
import importlib
import os

R_MIN_VERSION = "2.9.4"

R_SUBMODULES = [
    "rpy2.robjects.packages",
    "rpy2.situation",
    "rpy2.robjects.vectors",
]

R_SUBMODULES_3 = [
    "rpy2.rinterface_lib.callbacks",
]


class VersionError(BaseException):
    pass


class MockRPackage:
    def __init__(self, exception):
        self.exception = exception

    def __getattr__(self, item):
        raise self.exception


def import_r_submodules():
    importlib.import_module("rpy2.situation")
    r_home = rpy2.situation.get_r_home()
    if r_home is not None:
        if os.environ.get("R_HOME", None) is None:
            # set R_HOME globally (https://github.com/rpy2/rpy2/issues/796)
            os.environ["R_HOME"] = r_home
        if rpy2_is_version_3:
            mods = R_SUBMODULES + R_SUBMODULES_3
        else:
            mods = R_SUBMODULES
        for sm in mods:
            importlib.import_module(sm)


try:
    rpy2 = importlib.import_module("rpy2")
    if LooseVersion(rpy2.__version__) < LooseVersion(R_MIN_VERSION):
        raise VersionError("Please install 'rpy2>={}'!".format(R_MIN_VERSION))
except ImportError:
    rpy2 = MockRPackage(
        ImportError("Please install 'rpy2>={}'!".format(R_MIN_VERSION)))
    rpy2_is_version_3 = False
except BaseException as e:
    rpy2 = MockRPackage(e)
    rpy2_is_version_3 = False
else:
    rpy2_is_version_3 = LooseVersion(rpy2.__version__) >= LooseVersion("3.0")
    import_r_submodules()



"""Load RT-DC datasets"""

import pathlib
import warnings

from .core import RTDCBase
from . import fmt_dict, fmt_dcor, fmt_hdf5, fmt_tdms, fmt_hierarchy


def check_dataset(path_or_ds):
    """deprecated, to not use"""
    warnings.warn("Please use dclab.rtdc_dataset.check.check_dataset!",
                  DeprecationWarning)
    from . import check  # avoid circular import
    return check.check_dataset(path_or_ds)


def load_file(path, identifier=None, **kwargs):
    path = pathlib.Path(path).resolve()
    for fmt in [fmt_hdf5.RTDC_HDF5, fmt_tdms.RTDC_TDMS]:
        if fmt.can_open(path):
            return fmt(path, identifier=identifier, **kwargs)
    else:
        raise ValueError("Unknown file format: '{}'".format(path.suffix))


def new_dataset(data, identifier=None, **kwargs):
    """Initialize a new RT-DC dataset

    Parameters
    ----------
    data:
        can be one of the following:

        - dict
        - .tdms file
        - .rtdc file
        - subclass of `RTDCBase`
          (will create a hierarchy child)
        - DCOR resource URL
    identifier: str
        A unique identifier for this dataset. If set to `None`
        an identifier is generated.
    kwargs: dict
        Additional parameters passed to the RTDCBase subclass

    Returns
    -------
    dataset: subclass of :class:`dclab.rtdc_dataset.RTDCBase`
        A new dataset instance
    """
    if isinstance(data, dict):
        return fmt_dict.RTDC_Dict(data, identifier=identifier, **kwargs)
    elif fmt_dcor.is_dcor_url(data):
        return fmt_dcor.RTDC_DCOR(data, identifier=identifier, **kwargs)
    elif isinstance(data, RTDCBase):
        return fmt_hierarchy.RTDC_Hierarchy(data, identifier=identifier,
                                            **kwargs)
    elif isinstance(data, (pathlib.Path, str)):
        if not pathlib.Path(data).exists():
            raise FileNotFoundError("Could not find file '{}'!".format(data))
        else:
            return load_file(data, identifier=identifier, **kwargs)
    else:
        msg = "data type not supported: {}".format(data.__class__)
        raise NotImplementedError(msg)
